const express = require('express')
const googleapis = require('googleapis')
const jwtdecode = require('jwt-decode')
const AccountRepository = require('../repository/account')
const TokenRepository = require('../repository/token')

const OAuth2 = googleapis.auth.OAuth2

module.exports = function (db, oauthinfo) {
  const oauth = new OAuth2(oauthinfo.clientId, oauthinfo.clientSecret, oauthinfo.redirectUrl)
  const router = express.Router()
  const accountRepo = new AccountRepository(db, oauthinfo)
  const tokenRepo = new TokenRepository(db)

  router.route('/provider')

  // reponse object contains information about oauth provider
  //  - name: name of oauth provider
  //  - url: endpoint for user of oauth provider accept provide access token
  .get(function (req, res) {
    res.json({
      name: 'google',
      url: oauth.generateAuthUrl({
        scope: oauthinfo.scope,
        approval_prompt: 'auto',
        access_type: 'offline'
      })
    })
  })

  // login by code returned from oauth provider
  // redirect to endpoint with access_token and refresh_token in url query
  router.route('/login')
  .get(function (req, res) {
    // exchange code with oauth provider for access token
    oauth.getToken(req.query.code, function (errToken, tokens) {
      if (errToken) {
        res.status(500).send({
          message: 'exchange code for token error'
        })
        return
      }

      // save account to internal database if it is not early exists
      var tokenDecoded = jwtdecode(tokens.id_token)
      var issuer = tokenDecoded.iss
      var subject = tokenDecoded.sub
      accountRepo.findByIssuer(issuer, subject, function (errRepo, item) {
        if (errRepo) {
          res.json({
            message: 'repository.findIssuser.error'
          })
          return
        }

        var url = 'http://localhost:9000/app/account/login.html' +
            '?access_token=' + tokens.access_token +
            '&expiry_date=' + tokens.expiry_date

        if (tokens.refresh_token) {
          tokenRepo.create(null, {
            access_token: tokens.access_token,
            refresh_token: tokens.refresh_token,
            expiry_date: tokens.expiry_date
          }, function () {
            // todo: handler error here
          })
        }

        // create new account in application database
        if (!item) {
          // retrieve information of user in provider database
          var reqClone = {
            headers: { authorization: tokens.access_token }
          }
          accountRepo.authenticateEx(reqClone, function (err, user) {
            if (err) {
              res.status(401).json({
                message: 'must login to perform this action'
              })
              return
            }

            // create new account
            var item_ = {
              issuer: issuer,
              subject: subject,
              language: user.language
            }
            accountRepo.create(item_, function (error) {
              if (error) {
                res.json({
                  message: 'repository.create.error',
                  data: error
                })
                return
              }

              // redirect to client with access token
              res.redirect(url)
            })
          })
        } else {
          // redirect to client with access token
          res.redirect(url)
        }
      })
    })
  })

  router.route('/token')
  .get(function (req, res) {
    const accessToken = req.headers['authorization']
    tokenRepo.findAccessToken(accessToken, function (errFind, token) {
      if (errFind) {
        res.status(500).json({
          message: 'internal service error. try later'
        })
        return
      }
      if (!token) {
        res.status(404).json({
          message: 'not found refresh token'
        })
        return
      }

      oauth.setCredentials({
        refresh_token: token.refresh_token
      })
      oauth.refreshAccessToken(function (errRefesh, tokens) {
        if (errRefesh || !tokens) {
          res.status(401).json({
            message: 'must login to refresh access token'
          })
          return
        }

        // save new refresh_token if it is specify
        if (tokens.refresh_token) {
          tokenRepo.create(accessToken, {
            access_token: tokens.accessToken,
            expiry_date: tokens.expiry_date,
            refresh_token: tokens.refresh_token
          }, function () {
            // todo: handler here
          })
        }

        res.json({
          access_token: tokens.accessToken,
          expiry_date: tokens.expiry_date
        })
      })
    })
  })

  return router
}
