const mongoose = require('mongoose')
const applySchedulerModel = require('../model/scheduler')

// scheduler repository
// using to query item in database
// params
//  - db <connection>: instance of mongoose connection
// return instance of repository
function SchedulerRepo (db) {
  const self = this

  self.db = db
  if (!db.models.scheduler) {
    applySchedulerModel(mongoose, db)
  }
  self.model = db.models.scheduler
}

// shortcut to repository prototype
// using to access to repositoy prototype more quickly
const prototype = SchedulerRepo.prototype

// create new item
// params
//  - items <array|object>: list of items or one item object
//  - callback <function>: function will call
//    params
//     - error <object>: error during process
prototype.create = function (items, callback) {
  this.model.create(items, function (err) {
    callback(err)
  })
}

// find item by selector
// params
//  - selector <object>: params to select items
//    - pageSize <number>: number of item for select
//    - pageIndex <number>: index of page
//  - callback <function>: function will called
//    params
//     - error <object>: error during process
//     - items <array>: list of item matched with selector
prototype.find = function (selector, callback) {
  const self = this
  var skip = (selector.pageIndex - 1) * selector.pageSize
  var options = {
    skip: skip,
    limit: selector.pageSize
  }

  self.model.find({}, null, options, function (error, items) {
    callback(error, items)
  })
}

// find item by id, id coresponse with _id field
// params
//  - id <string>: identity of item
//  - callback <function>: function will call
//    params
//       - error <object>: error return by process
//       - item <array>: list of item has found
prototype.findById = function (id, callback) {
  this.model.findById(id, function (error, item) {
    callback(error, item)
  })
}

// remove
prototype.remove = function (itemId, authorId, callback) {
  this.model.remove({_id: itemId, author: authorId}, function (err, result) {
    callback(err, result)
  })
}

// search item by keyword, keyword will match with fields below
//  - name
//  - element of tags
// params
//  - keyword <string>: keyword for search
//  - callback <function>: function will called
//    params
//     - error <object>: error return by process
//     - items <array>: list of items matched by search
prototype.search = function (keyword, selector, callback) {
  var skip = (selector.pageIndex - 1) * selector.pageSize
  var options = { skip: skip, limit: selector.pageSize }

  var filter = {}
  if (keyword !== '*') {
    filter['$or'] = [
      { name: { $regex: new RegExp(keyword, 'gi') } },
      { tags: { $elemMatch: { $regex: new RegExp(keyword, 'gi') } } }
    ]
  }
  var fields = {notes: 0, tasks: 0, tags: 0}
  this.model.find(filter, fields, options, function (error, items) {
    callback(error, items)
  })
}

// same as search, but match with account identity
// params
//  - itemIds: item identity which user use of other user
prototype.searchAuthor = function (accountId, itemIds, keyword, selector, callback) {
  var skip = (selector.pageIndex - 1) * selector.pageSize
  var options = {skip: skip, limit: selector.pageSize}
  var fields = {notes: 0, tasks: 0, tags: 0}
  itemIds = !itemIds ? [] : itemIds
  var filter = {$and: [ { $or: [
    {author: accountId},
    {_id: { $in: itemIds }}
  ]}]}

  if (keyword !== '*') {
    filter['$and'].push({$or: [
      { name: { $regex: new RegExp(keyword, 'gi') } },
      { tags: { $elemMatch: { $regex: new RegExp(keyword, 'gi') } } }
    ]})
  }

  this.model.find(filter, fields, options, function (error, items) {
    callback(error, items)
  })
}

// drop collection
// prams
//  - callback <function>: function will called
//    params
//     - error <object>: error return by process
prototype.drop = function (callback) {
  this.model.remove(callback)
}

// define module
module.exports = SchedulerRepo
