// express middleware to
//  - authenticate by google oauth-2
//  - authorize by identity of account and groups

const googleapis = require('googleapis')
const mongoose = require('mongoose')

const OAuth2 = googleapis.auth.OAuth2

const applyAccountModel = require('../model/account')

// authorization class
// it will use accounts collection in mongoose database, so do not use
// same of name
// params
//  - db: mongoose database use to store information of user
//  - clientId: google client identity
//  - clientSecret: google client secret
//  - redirectUrl: url will call when provider authenticate successful
function Auth (db, clientId, clientSecret, redirectUrl, scopes) {
  const self = this

  self._db = db
  self._clientId = clientId
  self._clientSecret = clientSecret
  self._redirectUrl = redirectUrl
  self._scopes = scopes

  if (!db.models.account) { applyAccountModel(mongoose, db) }
  self._model = db.models.account

  self.oauth = new OAuth2(clientId, clientSecret, redirectUrl)
}
const prototype = Auth.prototype

// authenticate through provider
// params
//  - accessToken: required, string will help identity account
//  - refreshToken: use to exchange new accessToken if it expired
//  - cb: function will be call after done
//    params
//      - err: error during process
//      - oauthAcc: information of account from provider
//      - newAccessToken: new access_token. if it not empty,
//        must be replace old accessToken by new accessToken
prototype._authenticate = function (accessToken, refreshToken, cb) {
  const self = this
  const plus = googleapis.plus('v1')

  self.oauth.setCredentials({
    access_token: accessToken,
    refresh_token: refreshToken
  })

  plus.people.get({userId: 'me', auth: self.oauth}, function (err, user) {
    cb(err, user, '?new_access_token')
  })
}

// express middleware
// verify request with user group. that mean
//  - request must be authenticate
//  - authenticated user must in group
// notes: only group allowd to meet flexible of api
// params
//  - opts
//    - group: required, user group which user must in
//    - msg_401: message responses to endpoint if not authenticate
//    - msg_403: message responses to endpoint if authenticate successful,
//      but not authorize
// behavior:
//  - if authorize successful, continue next route
//  - if authorize fails, response message to endpoint and terminate
prototype.authorize = function (opts) {
  const self = this

  return function (req, res, next) {
    self._authenticate(req.headers['authorization'],
    req.headers['refresh_token'], authCallback)

    function authCallback (err, oauthAcc, newAccessToken) {
      if (err) {
        res.status(500).json({
          message: 'internal service error. try later'
        })
        return
      }
      if (!oauthAcc) {
        res.status(401).json({
          message: opts.msg_401
        })
      }

      // find account in database
      self._model.findOne({subject: oauthAcc.id}, findCallback)

      function findCallback (err, appAcc) {
        if (err) {
          res.status(500).json({
            message: 'internal service error. try later'
          })
          return
        }

        if (!appAcc) {
          res.status(401).json({message: opts.msg_401})
          return
        }

        // verify group
        if (opts.group) {
          if (!appAcc.groups || appAcc.groups.indexOf(opts.group) < 0) {
            res.status(403).json({message: opts.msg_403})
            return
          }
        }

        // assign user to request
        req.oauthAcc = oauthAcc
        req.appAcc = appAcc

        next()
      }
    }
  }
}

module.exports = Auth
